/*============================================	
  GApplication.cpp
  
  1996 Michael Stochosky, see BeVGB.doc for
  details.
  ============================================*/


#ifndef _RECORD_H
#include <Record.h>
#endif

#ifndef G_WINDOW_H
#include "GWindow.h"
#endif

#ifndef G_PREFERENCES_H
#include "GPreferences.h"
#endif

#ifndef G_VIEW_H
#include "GView.h"
#endif

#ifndef G_APPLICATION_H
#include "GApplication.h"
#endif

// emulator communication ports
port_id gOutputPort;
port_id gInputPort;


// main entry point
main()
{	
	GApplication *myApplication;
	
	// open emulator I/O ports
	gOutputPort = create_port(1, "gb_emulator_output");
	gInputPort = create_port(1, "gb_emulator_input");
	
	// start application
	myApplication = new GApplication();
	myApplication->Run();
	
	delete(myApplication);
	return(0);
}



// create application
GApplication::GApplication()
		  		  : BApplication(tSIGNATURE)
{	
	// create main window w/game boy screen demensions
	mWindow = new GWindow(160, 144);
	
	// file open variables
	mFrame.Set(0.0, 0.0, 0.0, 0.0);
	mDirectory.record = -1;
	mDirectory.database = -1;
	strcpy(mMarked, "");
}


// init application
void GApplication::ReadyToRun(void)
{
	// WARNING!! Remember to change references to
	// indexs of item in MenuWillShow() if order changes
	
	// construct main menu
	BMenuItem *item;
	BPopUpMenu *menu = new BPopUpMenu("", FALSE, FALSE);
	BMenu *submenu;
	BMessage *msg;
	
	item = new BMenuItem("About BeVGB...", new BMessage(B_ABOUT_REQUESTED));	// 0
	item->SetTarget(be_app);
	menu->AddItem(item);
	
	item = new BMenuItem("Preferences",new BMessage(mSETTINGS), ';');			// 1
	item->SetTarget(be_app);
	menu->AddItem(item);
	
	menu->AddSeparatorItem();													// 2
	
	item = new BMenuItem("Open", new BMessage(mOPEN), 'O');						// 3
	item->SetTarget(be_app);
	menu->AddItem(item);
	
	item = new BMenuItem("Pause", new BMessage(mPAUSE), 'P');					// 4
	item->SetTarget(mWindow);		// target window
	menu->AddItem(item);
	
	item = new BMenuItem("Reset", new BMessage(mRESET), 'R');					// 5
	item->SetTarget(mWindow);		// target window
	menu->AddItem(item);
	
	menu->AddSeparatorItem();													// 6
	
		// colors submenu
		submenu = new BMenu("Colors");
		submenu->SetRadioMode(true);
		
		msg = new BMessage(mGB_FLAGS);
		msg->AddLong("palette", 0);
		item = new BMenuItem("Grayscale", msg);
		item->SetMarked(true);
		item->SetTarget(mWindow);
		submenu->AddItem(item);
		
		msg = new BMessage(mGB_FLAGS);
		msg->AddLong("palette", 1);
		item = new BMenuItem("Spinach Green", msg);
		item->SetTarget(mWindow);
		submenu->AddItem(item);
		
		msg = new BMessage(mGB_FLAGS);
		msg->AddLong("palette", 2);
		item = new BMenuItem("Debug", msg);
		item->SetTarget(mWindow);
		submenu->AddItem(item);
		
		item = new BMenuItem(submenu);											// 7
		menu->AddItem(item);
			
	menu->AddSeparatorItem();													// 8
	
	item = new BMenuItem("Quit", new BMessage(B_QUIT_REQUESTED), 'Q');			// 9
	item->SetTarget(be_app);
	menu->AddItem(item);
	
	// attach to app
	be_app->SetMainMenu(menu);
	
	// start pumping
	PostMessage(mREFRESH);
}


void GApplication::MessageReceived(BMessage *msg)
{		
	if(msg->what != 'draw')
	msg->PrintToStream();
	
	// sleep a little
	snooze(SLEEP_TIME);
	
	switch (msg->what) {
		
		case mREFRESH:
			mWindow->PostMessage(mREFRESH);
			PostMessage(mREFRESH);
			break;
		
		// show the preferences panel
		case mSETTINGS:
			if( CountWindows() == 1 )	// 2 windows means preferences are open
				GPreferences *win = new GPreferences();
			else
				;	// window already open, activate it
			break;
		
		// open a new cartridge for execution	
		case mOPEN:
			// file panel isn't running
			if( IsFilePanelRunning() == false )
			{
				BMessage *open = new BMessage(B_REFS_RECEIVED);
				
				// build message
				open->AddString("filter", "Cart images");
				open->AddString("filter", "All files");
				
				open->AddLong("Cart images", tCART);
				open->AddLong("All files", 0);
				
				if( mFrame.Width() > 0 & mFrame.Height() > 0 )
					open->AddRect("frame", mFrame);
				
				if( mDirectory.database >= 0 & mDirectory.record >= 0 )
					open->AddRef("directory", mDirectory);
				
				if( strlen(mMarked) > 0 )
					open->AddString("marked", mMarked);
				
				RunFilePanel("Open Cartridge", "Open", "Cancel", false, open);
			}
			break;
		
		// we didn't handle it, send back a response
		default:
			inherited::MessageReceived(msg);
			break;
	}
}


void GApplication::AboutRequested(void) {
	BAlert				*my_alert;

// poor mans about box
	my_alert = new BAlert("",	"BeVGB 1.0\n\n"\
								"by Michael Stochosky\n"\
								"origninal code by Marat Fayzullin\n"\
								"inspired by John Stiles's Macintosh version.", "narf");
	my_alert->Go();
}


// menus about to show, adjust settings
void GApplication::MenusWillShow(void)
{
	BPopUpMenu *menu = MainMenu();
	BMenuItem *item;
	
	// pause menu item
	if( mWindow->IsPaused() )
	{
		item = menu->ItemAt(4);			// index of "Pause" item
		item->SetLabel("Resume");
	}
	else
	{
		item = menu->ItemAt(4);			// index of "Pause" item
		item->SetLabel("Pause");
	
	}
}


// file panel has closed, init game if applicable
void GApplication::FilePanelClosed(BMessage *msg)
{
	// record file panel position, directory, etc.
	mFrame = msg->FindRect("frame");
	mDirectory = msg->FindRef("directory");
	strcpy(mMarked, msg->FindString("marked"));
}


// recieved a ref, close panel
void GApplication::RefsReceived(BMessage *msg)
{
	ulong type;
	long count;

	msg->PrintToStream();
	
	// get first item in file list
	msg->GetInfo("refs", &type, &count);
	if( count > 0 )
	{
		record_ref item = msg->FindRef("refs", count-1);
		if( item.database >= 0 & item.record >= 0 )
		{
			if( does_ref_conform(item, "File") )
			{
				// tell window to open emulator with this cartridge
				BMessage *win_msg = new BMessage(mRUN_CART);
				win_msg->AddRef("cart", item);
				mWindow->PostMessage(win_msg);
			}
		}	
	}
}