/*============================================	
  GWindow.cpp
  
  1996 Michael Stochosky, see BeVGB.doc for
  details.
  ============================================*/

#ifndef _APPLICATION_H
#include <Application.h>
#endif

#ifndef G_APPLICATION_H
#include "GApplication.h"
#endif

#ifndef G_WINDOW_H
#include "GWindow.h"
#endif


GWindow::GWindow(int width, int height)
				: BWindow(BRect(0, 0, width-1, height-1),\
					"BeVGB", B_TITLED_WINDOW,\
					B_CURRENT_WORKSPACE )
{
	float			dockwidth;
	
	Lock();
	
	// move window into place
	if( get_dock_width(&dockwidth) == B_ERROR )
		dockwidth = 70.0;
	MoveTo( dockwidth, 40.0 );
	
	// set resizing and other info
#ifndef DEBUG
	SetDiscipline(false);
#endif
	// BeOS BUG: Minimum coordinates, not widths and heigths
	SetSizeLimits(160-1, (160*2)-1, 144-1, (144*2)-1);
	
	// set up a rectangle and instantiate a new view
	BRect rect(0.0, 0.0, width-1, height-1);
	rect.OffsetTo(B_ORIGIN);
	mView = new GView(rect, "GView");
	
	// add view to window
	AddChild(mView);
	
	// make window visible
	Show();
	
	Unlock();
}


// process redraw messages
void GWindow::MessageReceived(BMessage *msg)
{	
	Lock();
	
	switch (msg->what) {
		
		// update the screen
		case mREFRESH:
			// Prevent messages from piling up
			BMessage *msg2;
			MessageQueue()->Lock();
			while ((msg2 = MessageQueue()->FindMessage(mREFRESH, 0)) != NULL)
				MessageQueue()->RemoveMessage(msg2);
			MessageQueue()->Unlock();
			
			// get port message
			long type, length;
			char *bits = (char*)malloc(WIDTH*HEIGHT);
			if( bits )
			{
				length = read_port_etc(gOutputPort, &type, bits, WIDTH*HEIGHT, B_TIMEOUT, 5000.0);
				if( (length == WIDTH*HEIGHT) && type == B_RAW_TYPE )
					mView->SetBits(bits, length);
				
				free(bits);
			}
			
			// draw changed buffer & send keyboard
			if( !IsPaused() )
			{
				SendJoyState();
				mView->Draw(mView->Bounds());
			}
			
			break;

		// reinit the emulator with a new cartridge
		case mRUN_CART:
			record_ref ref;
			
			ref = msg->FindRef("cart");
			mView->Run(ref);
			break;
		
		// pause or resume execution of the emulator thread
		case mPAUSE:
			
			if( IsPaused() )
				mView->Resume();
			else
				mView->Pause();
			break;
		
		// reset the emulator with the current cartridge
		case mRESET:
			mView->Reset();
			break;
		
		// pass GB messages on to view
		case mGB_FLAGS:
			mView->MessageReceived(msg);
			break;
		
		// handle files dropped from browser
		case B_SIMPLE_DATA:
			if( msg->WasDropped() && msg->HasRef("refs") )
			{
				BFile file;
				ulong type, app;
				record_ref ref;
		
				// get the file type
				ref = msg->FindRef("refs");
				file.SetRef(ref);
				file.GetTypeAndApp(&type, &app);
				
				// only open files of type 'CART
				if( type == tCART )
				{
					BMessage *cart = new BMessage(mRUN_CART);
					
					// send to us!
					cart->AddRef( "cart", ref );
					PostMessage(cart);
				}
			}
			break;
		
		default:
			inherited::MessageReceived(msg);
	}
	
	Unlock();
}


// is the emulator paused?
bool GWindow::IsPaused(void)
{
	return mView->IsPaused();
}


// send the state of the keys to the emulator
void GWindow::SendJoyState()
{	
	uchar JoyState = 0xFF;
	key_info info;
	long err;

	// key code selector
	#define KEY( code )	(info.key_states[(code)>>3] & (1<<(7-((code)%8))))
	
	// key scan codes
	#define K_START		0x7F
	#define K_SELECT	0xBF
	#define K_B			0xDF
	#define K_A			0xEF
	#define K_DOWN		0xF7
	#define K_UP		0xFB
	#define K_LEFT		0xFD
	#define K_RIGHT		0xFE
 
 	// scan keys
 	err = mView->GetKeys( &info, false );
	if( err == B_NO_ERROR )
	{
		// left shift - right shift - Return
		if( KEY(0x4B) || KEY(0x56) || KEY(0x47) )
			JoyState &= (K_START); else JoyState |= (~K_START);
		
		// left ctl - right ctl - tab - left alt
		if( KEY(0x5C) || KEY(0x60) || KEY(0x26) || KEY(0x5D) )
			JoyState &= (K_SELECT); else JoyState |= (~K_SELECT);
		
		// a -> h
		if( KEY(0x3c) || KEY(0x3e) || KEY(0x3d) || KEY(0x3f) || KEY(0x40) || KEY(0x41) )
			JoyState &= (K_A); else JoyState |= (~K_A);
		
		// z -> n
		if( KEY(0x4c) || KEY(0x4e) || KEY(0x4d) || KEY(0x4f) || KEY(0x50) || KEY(0x51) )
			JoyState &= (K_B); else JoyState |= (~K_B);
		
		// down, up, left, and right arrows
		KEY(0x62) ? JoyState &= (K_DOWN) : JoyState |= (~K_DOWN);
		KEY(0x57) ? JoyState &= (K_UP) : JoyState |= (~K_UP);
		KEY(0x61) ? JoyState &= (K_LEFT) : JoyState |= (~K_LEFT);
		KEY(0x63) ? JoyState &= (K_RIGHT) : JoyState |= (~K_RIGHT);
	}// no err
	
	// send JoyState to emulator
	char *buf = (char*)malloc(2);
	buf[0] = JoyState;	// keys
	buf[1] = false;		// don't quit
	write_port_etc(gInputPort, B_RAW_TYPE, buf, 2, B_TIMEOUT, 0.0);
	if( buf ) free(buf);
}


// request to quit
bool GWindow::QuitRequested()
{
	mView->Sync();

	be_app->PostMessage(B_QUIT_REQUESTED);
	return(TRUE);
}